@@ -9,7 +9,7 @@ use SQL::ReservedWords::MySQL();
 
 use Rose::DB;
 
-our $VERSION = '0.721';
+our $VERSION = '0.731';
 
 our $Debug = 0;
 
@@ -35,6 +35,13 @@ sub build_dsn
 
 sub dbi_driver { 'mysql' }
 
+sub mysql_auto_reconnect { shift->dbh_attribute_boolean('mysql_auto_reconnect', @_) }
+sub mysql_enable_utf8    { shift->dbh_attribute_boolean('mysql_enable_utf8', @_) }
+sub mysql_use_result     { shift->dbh_attribute_boolean('mysql_use_result', @_) }
+
+
+sub dbh_attributes { qw(mysql_auto_reconnect mysql_use_result mysql_enable_utf8) }
+
 sub database_version
 {
   my($self) = shift;
@@ -220,6 +227,42 @@ sub select_bitfield_column_sql
   }
 }
 
+sub parse_set
+{
+  my($self) = shift;
+  
+  return $_[0]  if(ref $_[0] eq 'ARRAY');
+  return [ @_ ] if(@_ > 1);
+  
+  my $val = $_[0];
+  
+  return undef  unless(defined $val);
+  
+  my @set = split(/,/, $val);
+  
+  return \@set;
+}
+
+sub format_set
+{
+  my($self) = shift;
+
+  my @set = (ref $_[0]) ? @{$_[0]} : @_;
+
+  return undef  unless(@set && defined $set[0]);
+
+  return join(',', map 
+  {
+    if(!defined $_)
+    {
+      Carp::croak 'Undefined value found in array or list passed to ',
+                  __PACKAGE__, '::format_set()';
+    }
+    else { $_ }
+  }
+  @set);
+}
+
 sub refine_dbi_column_info
 {
   my($self, $col_info) = @_;
@@ -247,12 +290,18 @@ sub refine_dbi_column_info
     }
   }
 
-  # Put valid enum values in standard key
-  if($col_info->{'TYPE_NAME'} eq 'enum')
+  # Put valid SET and ENUM values in standard keys
+  if($col_info->{'TYPE_NAME'} eq 'set')
+  {
+
+    $col_info->{'RDBO_SET_VALUES'} = $col_info->{'mysql_values'};
+  }
+  elsif($col_info->{'TYPE_NAME'} eq 'enum')
   {
     $col_info->{'RDBO_ENUM_VALUES'} = $col_info->{'mysql_values'};
   }
 
+
   return;
 }
 
@@ -373,6 +422,34 @@ MySQL does not have a native "interval" data type, but this data type can be emu
 
 =head1 OBJECT METHODS
 
+=over 4
+
+=item B<mysql_auto_reconnect [BOOL]>
+
+Get or set the L<mysql_auto_reconnect|DBD::mysql/mysql_auto_reconnect> database handle attribute.  This is set directly on the L<dbh|Rose::DB/dbh>, if one exists.  Otherwise, it will be set when the L<dbh|Rose::DB/dbh> is created.  If no value for this attribute is defined (the default) then it will not be set when the L<dbh|Rose::DB/dbh> is created, deferring stead to whatever default value L<DBD::mysql> chooses.
+
+Returns the value of this attribute in the L<dbh|Rose::DB/dbh>, if one exists, or the value that will be set when the L<dbh|Rose::DB/dbh> is next created.
+
+See the L<DBD::mysql|DBD::mysql/mysql_use_result> documentation to learn more about this attribute.
+
+=item B<mysql_enable_utf8 [BOOL]>
+
+Get or set the L<mysql_enable_utf8|DBD::mysql/mysql_enable_utf8> database handle attribute.  This is set directly on the L<dbh|Rose::DB/dbh>, if one exists.  Otherwise, it will be set when the L<dbh|Rose::DB/dbh> is created.  If no value for this attribute is defined (the default) then it will not be set when the L<dbh|Rose::DB/dbh> is created, deferring stead to whatever default value L<DBD::mysql> chooses.
+
+Returns the value of this attribute in the L<dbh|Rose::DB/dbh>, if one exists, or the value that will be set when the L<dbh|Rose::DB/dbh> is next created.
+
+See the L<DBD::mysql|DBD::mysql/mysql_use_result> documentation to learn more about this attribute.
+
+=item B<mysql_use_result [BOOL]>
+
+Get or set the L<mysql_use_result|DBD::mysql/mysql_use_result> database handle attribute.  This is set directly on the L<dbh|Rose::DB/dbh>, if one exists.  Otherwise, it will be set when the L<dbh|Rose::DB/dbh> is created.  If no value for this attribute is defined (the default) then it will not be set when the L<dbh|Rose::DB/dbh> is created, deferring stead to whatever default value L<DBD::mysql> chooses.
+
+Returns the value of this attribute in the L<dbh|Rose::DB/dbh>, if one exists, or the value that will be set when the L<dbh|Rose::DB/dbh> is next created.
+
+See the L<DBD::mysql|DBD::mysql/mysql_use_result> documentation to learn more about this attribute.
+
+=back
+
 =head2 Value Parsing and Formatting
 
 =over 4
@@ -389,6 +466,10 @@ Given a L<DateTime::Duration> object, return a string formatted according to the
 
 If the resulting string is longer than L<max_interval_characters|/max_interval_characters>, a fatal error will occur.
 
+=item B<format_set ARRAYREF | LIST>
+
+Given a reference to an array or a list of values, return a string formatted according to the rules of MySQL's "SET" data type.  Undef is returned if ARRAYREF points to an empty array or if LIST is not passed.  If th array or list contains undefined values, a fatal error will occur.
+
 =item B<parse_array STRING | LIST | ARRAYREF>
 
 Parse STRING and return a reference to an array.  STRING should be formatted according to the MySQL array data type emulation format returned by L<format_array()|/format_array>.  Undef is returned if STRING is undefined.
@@ -403,6 +484,14 @@ Parse STRING and return a L<DateTime::Duration> object.  STRING should be format
 
 If STRING is a L<DateTime::Duration> object, a valid interval keyword (according to L<validate_interval_keyword|Rose::DB/validate_interval_keyword>), or if it looks like a function call (matches C</^\w+\(.*\)$/>) then it is returned unmodified.  Otherwise, undef is returned if STRING could not be parsed as a valid "interval" value.
 
+=item B<parse_set STRING | LIST | ARRAYREF>
+
+Parse STRING and return a reference to an array.  STRING should be formatted according to MySQL's "SET" data type.  Undef is returned if STRING is undefined.
+
+If a LIST of more than one item is passed, a reference to an array containing the values in LIST is returned.
+
+If a an ARRAYREF is passed, it is returned as-is.
+
 =item B<validate_date_keyword STRING>
 
 Returns true if STRING is a valid keyword for the MySQL "date" data type.  Valid date keywords are:
@@ -436,6 +525,6 @@ John C. Siracusa (siracusa@mindspring.com)
 
 =head1 COPYRIGHT
 
-Copyright (c) 2006 by John C. Siracusa.  All rights reserved.  This program is
+Copyright (c) 2007 by John C. Siracusa.  All rights reserved.  This program is
 free software; you can redistribute it and/or modify it under the same terms
 as Perl itself.
